@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012, 2022, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2012, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -234,6 +234,7 @@ public class CurrencyNames_cs extends OpenListResourceBundle {
             { "MNT", "MNT" },
             { "MOP", "MOP" },
             { "MRO", "MRO" },
+            { "MRU", "MRU" },
             { "MTL", "MTL" },
             { "MTP", "MTP" },
             { "MUR", "MUR" },
@@ -319,6 +320,7 @@ public class CurrencyNames_cs extends OpenListResourceBundle {
             { "UZS", "UZS" },
             { "VEB", "VEB" },
             { "VEF", "VEF" },
+            { "VES", "VES" },
             { "VND", "VND" },
             { "VNN", "VNN" },
             { "VUV", "VUV" },
@@ -615,7 +617,8 @@ public class CurrencyNames_cs extends OpenListResourceBundle {
             { "uyu", "uruguaysk\u00e9 peso" },
             { "uzs", "uzbeck\u00fd sum" },
             { "veb", "venezuelsk\u00fd bol\u00edvar (1871\u20132008)" },
-            { "vef", "venezuelsk\u00fd bol\u00edvar" },
+            { "vef", "venezuelsk\u00fd bol\u00edvar (2008\u20132018)" },
+            { "ves", "venezuelsk\u00fd bol\u00edvar" },
             { "vnd", "vietnamsk\u00fd dong" },
             { "vnn", "vietnamsk\u00fd dong (1978\u20131985)" },
             { "vuv", "vanuatsk\u00fd vatu" },
